#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include "signer.h"

static size_t
WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
    printf("%.*s", size * nmemb, (char*)contents);
    return size * nmemb;
}

int main(void)
{
    //initialize signature params
    sig_params_t params;
    sig_params_init(&params);
    // 认证用的ak和sk硬编码到代码中或者明文存储都有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
    // 本示例以ak和sk保存在环境变量中为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK和HUAWEICLOUD_SDK_SK。
    sig_str_t app_key = sig_str(getenv("HUAWEICLOUD_SDK_AK"));
    sig_str_t app_secret = sig_str(getenv("HUAWEICLOUD_SDK_SK"));
    sig_str_t host = sig_str("30030113-3657-4fb6-a7ef-90764239b038.apigw.cn-north-1.huaweicloud.com");
    sig_str_t method = sig_str("GET");
    sig_str_t uri = sig_str("/app1");
    sig_str_t query_str = sig_str("a=1&b=2");
    sig_str_t payload = sig_str("");
    params.key = app_key;
    params.secret = app_secret;
    params.host = host;
    params.method = method;
    params.uri = uri;
    params.query_str = query_str;
    params.payload = payload;
    sig_headers_add(&params.headers, "Content-Type", "application/json");
    //generate signature. generated Authorization header is inserted into params.headers
    if (sig_sign(&params) != 0) {
        printf("sign failed\n");
        sig_params_free(&params);
        return -1;
    }

    //send http request using curl library
    curl_global_init(CURL_GLOBAL_ALL);
    CURL *curl = curl_easy_init();
    CURLcode res;

    curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, params.method.data);
    char url[1024];
    sig_snprintf(url, 1024, "http://%V%V?%V", &params.host, &params.uri, &params.query_str);
    curl_easy_setopt(curl, CURLOPT_URL, url);
    struct curl_slist *chunk = NULL;
    for (int i = 0; i < params.headers.len; i++) {
        char header[1024];
        sig_snprintf(header, 1024, "%V: %V", &params.headers.data[i].name, &params.headers.data[i].value);
        printf("%s\n", header);
        chunk = curl_slist_append(chunk, header);
    }
    printf("-------------\n");
    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, chunk);
    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, params.payload.data);
    curl_easy_setopt(curl, CURLOPT_NOBODY, 0L);
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
    //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
    res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
    }
    else {
        long status;
        curl_easy_getinfo(curl, CURLINFO_HTTP_CODE, &status);
        printf("\nstatus %d\n", status);
    }
    curl_easy_cleanup(curl);

    curl_global_cleanup();

    //free signature params
    sig_params_free(&params);

    return 0;
}