#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include "signer_common.h"

int sig_empty_str(sig_str_t* str)
{
    return (str == NULL || str->data == NULL || str->len == 0) ? 1 : 0;
}

void sig_trim_str(sig_str_t* str, sig_str_t* dst)
{
    char* start = str->data;
    while (*start == ' ')
    {
        start++;
    }
    char* end = str->data + str->len - 1;
    while (*end == ' ')
    {
        end--;
    }
    dst->data = start;
    dst->len = end - start + 1;
}

/* reference: http://lxr.nginx.org/source/src/core/ngx_string.c */
static int sig_vslprintf(char* buf, char* last, const char* fmt, va_list args)
{
    char*    c_ptr = buf;
    sig_str_t*  str;

    while (*fmt && c_ptr < last)
    {
        size_t n_max = last - c_ptr;
        if (*fmt == '%')
        {
            if (*(fmt + 1) == 'V')
            {
                str = va_arg(args, sig_str_t *);
                if (sig_empty_str(str))
                {
                    goto finished;
                }
                size_t cp_len = n_max >= str->len ? str->len : n_max;
                strncpy(c_ptr, str->data, cp_len);
                c_ptr += cp_len;
                fmt += 2;
            }
            else if(*(fmt + 1) == 'v') {
                str = va_arg(args, sig_str_t *);
                if (sig_empty_str(str))
                {
                    goto finished;
                }
                size_t cp_len = n_max >= str->len ? str->len : n_max;
                for (size_t cp_i = 0; cp_i < cp_len; cp_i++) {
                    c_ptr[cp_i] = tolower(str->data[cp_i]);
                }
                c_ptr += cp_len;
                fmt += 2;
            }
            else
            {
                *(c_ptr++) = *(fmt++);
            }
        }
        else
        {
            *(c_ptr++) = *(fmt++);
        }
    }
finished:
    *c_ptr = '\0';
    return c_ptr - buf;
}

int sig_sprintf(char* buf, const char* fmt, ...)
{
    int len = 0;
    va_list args;
    va_start(args, fmt);
    len = sig_vslprintf(buf, (void*)-1, fmt, args);
    va_end(args);
    return len;
}

int sig_snprintf(char* buf, unsigned int n, const char* fmt, ...)
{
    int len = 0;
    va_list args;
    va_start(args, fmt);
    len = sig_vslprintf(buf, buf + n, fmt, args);
    va_end(args);
    return len;
}


size_t sig_unescape(char *dst, char *s, size_t size)
{
    char ch, c, decoded;
    char *d = dst;
    enum {
        sw_usual = 0,
        sw_quoted,
        sw_quoted_second
    } state;

    state = 0;
    decoded = 0;
    while (size--) {

        ch = *s++;

        switch (state) {
        case sw_usual:

            if (ch == '%') {
                state = sw_quoted;
                break;
            }

            if (ch == '+') {
                *d++ = ' ';
                break;
            }

            *d++ = ch;
            break;

        case sw_quoted:

            if (ch >= '0' && ch <= '9') {
                decoded = (unsigned char)(ch - '0');
                state = sw_quoted_second;
                break;
            }

            c = (unsigned char)(ch | 0x20);
            if (c >= 'a' && c <= 'f') {
                decoded = (unsigned char)(c - 'a' + 10);
                state = sw_quoted_second;
                break;
            }

            /* the invalid quoted character */

            state = sw_usual;

            *d++ = ch;

            break;

        case sw_quoted_second:

            state = sw_usual;

            if (ch >= '0' && ch <= '9') {
                ch = (unsigned char)((decoded << 4) + ch - '0');

              
                *d++ = ch;

                break;
            }

            c = (unsigned char)(ch | 0x20);
            if (c >= 'a' && c <= 'f') {
                ch = (unsigned char)((decoded << 4) + c - 'a' + 10);

                *d++ = ch;

                break;
            }

            /* the invalid quoted character */

            break;
        }
    }
    return d - dst;

}


size_t sig_escape(char *dst, char *src, size_t size, int type)
{
    char *   d = dst;
    static char   hex[] = "0123456789ABCDEF";

    while (size) {
        if (isalnum((unsigned char)(*src)) || *src == '.' || *src == '~'
            || *src == '_' || *src == '-' || (type == SIG_ESCAPE_URI && *src == '/') || (type == SIG_ESCAPE_QUERY && *src == '=')) {
            *d++ = *src++;
        }
        else {
            *d++ = '%';
            *d++ = hex[*(unsigned char*)src >> 4];
            *d++ = hex[*(unsigned char*)src & 0xf];
            src++;
        }
        size--;
    }

    return d - dst;
}